@@ -181,13 +181,32 @@ auth_message_handler (GSAuthMessageStyle style,
 static gboolean
 gs_auth_queued_message_handler (GsAuthMessageHandlerData *data)
 {
+        gboolean res;
+
+        if (gs_auth_get_verbose ()) {
+                g_message ("Waiting for lock");
+        }
+
         g_mutex_lock (message_handler_mutex);
-        data->should_interrupt_stack = data->closure->cb_func (data->style,
-                                                               data->msg,
-                                                               data->resp,
-                                                               data->closure->cb_data) == FALSE;
+
+        if (gs_auth_get_verbose ()) {
+                g_message ("Waiting for response");
+        }
+
+        res = data->closure->cb_func (data->style,
+                                      data->msg,
+                                      data->resp,
+                                      data->closure->cb_data);
+
+        data->should_interrupt_stack = res == FALSE;
+
         g_cond_signal (message_handled_condition);
         g_mutex_unlock (message_handler_mutex);
+
+        if (gs_auth_get_verbose ()) {
+                g_message ("Got response");
+        }
+
         return FALSE;
 }
 
@@ -211,12 +230,20 @@ gs_auth_run_message_handler (struct pam_closure *c,
          */
         g_idle_add ((GSourceFunc) gs_auth_queued_message_handler, &data);
 
+        if (gs_auth_get_verbose ()) {
+                g_message ("Waiting for respose to message style %d: '%s'", style, msg);
+        }
+
         /* Wait for the response
          */
         g_cond_wait (message_handled_condition,
                      message_handler_mutex);
         g_mutex_unlock (message_handler_mutex);
 
+        if (gs_auth_get_verbose ()) {
+                g_message ("Got respose to message style %d: interrupt:%d", style, data.should_interrupt_stack);
+        }
+
         return data.should_interrupt_stack == FALSE;
 }
 
@@ -275,6 +302,10 @@ pam_conversation (int                        nmsgs,
                                       NULL);
 
                 if (c->cb_func != NULL) {
+                        if (gs_auth_get_verbose ()) {
+                                g_message ("Handling message style %d: '%s'", style, utf8_msg);
+                        }
+
  			/* blocks until the gui responds
   			 */
   			res = gs_auth_run_message_handler (c,
@@ -282,6 +313,10 @@ pam_conversation (int                        nmsgs,
   							   utf8_msg,
   							   &reply [replies].resp);
 
+                        if (gs_auth_get_verbose ()) {
+                                g_message ("Msg handler returned %d", res);
+                        }
+
                         /* If the handler returns FALSE - interrupt the PAM stack */
                         if (res) {
                                 reply [replies].resp_retcode = PAM_SUCCESS;
@@ -443,9 +478,83 @@ gs_auth_thread_func (int auth_operation_fd)
 {
         static const int flags = 0;
         int              status;
+        int              status2;
+        struct timespec  timeout;
+        sigset_t         set;
+        const void      *p;
+
+        timeout.tv_sec = 0;
+        timeout.tv_nsec = 1;
+
+        set = block_sigchld ();
 
         status = pam_authenticate (pam_handle, flags);
 
+        sigtimedwait (&set, NULL, &timeout);
+        unblock_sigchld ();
+
+        if (gs_auth_get_verbose ()) {
+                g_message ("   pam_authenticate (...) ==> %d (%s)",
+                           status,
+                           PAM_STRERROR (pam_handle, status));
+        }
+
+        if (status != PAM_SUCCESS) {
+                goto done;
+        }
+
+        if ((status = pam_get_item (pam_handle, PAM_USER, &p)) != PAM_SUCCESS) {
+                /* is not really an auth problem, but it will
+                   pretty much look as such, it shouldn't really
+                   happen */
+                goto done;
+        }
+
+        /* We don't actually care if the account modules fail or succeed,
+         * but we need to run them anyway because certain pam modules
+         * depend on side effects of the account modules getting run.
+         */
+        status2 = pam_acct_mgmt (pam_handle, 0);
+
+        if (gs_auth_get_verbose ()) {
+                g_message ("pam_acct_mgmt (...) ==> %d (%s)\n",
+                           status2,
+                           PAM_STRERROR (pam_handle, status2));
+        }
+
+        /* FIXME: should we handle these? */
+        switch (status2) {
+        case PAM_SUCCESS:
+                break;
+        case PAM_NEW_AUTHTOK_REQD:
+                break;
+        case PAM_AUTHINFO_UNAVAIL:
+                break;
+        case PAM_ACCT_EXPIRED:
+                break;
+        case PAM_PERM_DENIED:
+                break;
+        default :
+                break;
+        }
+
+        /* Each time we successfully authenticate, refresh credentials,
+           for Kerberos/AFS/DCE/etc.  If this fails, just ignore that
+           failure and blunder along; it shouldn't matter.
+
+           Note: this used to be PAM_REFRESH_CRED instead of
+           PAM_REINITIALIZE_CRED, but Jason Heiss <jheiss@ee.washington.edu>
+           says that the Linux PAM library ignores that one, and only refreshes
+           credentials when using PAM_REINITIALIZE_CRED.
+        */
+        status2 = pam_setcred (pam_handle, PAM_REINITIALIZE_CRED);
+        if (gs_auth_get_verbose ()) {
+                g_message ("   pam_setcred (...) ==> %d (%s)",
+                           status2,
+                           PAM_STRERROR (pam_handle, status2));
+        }
+
+ done:
         /* we're done, close the fd and wake up the main
          * loop
          */
@@ -465,8 +574,8 @@ gs_auth_loop_quit (GIOChannel  *source,
 }
 
 static gboolean
-gs_auth_identify_user (pam_handle_t *handle,
-		       int          *status)
+gs_auth_pam_verify_user (pam_handle_t *handle,
+                         int          *status)
 {
         GThread    *auth_thread;
         GIOChannel *channel;
@@ -558,13 +667,9 @@ gs_auth_verify_user (const char       *username,
                      GError          **error)
 {
         int                status = -1;
-        int                status2;
         struct pam_conv    conv;
         struct pam_closure c;
-        sigset_t           set;
-        struct timespec    timeout;
         struct passwd     *pwent;
-        const void        *p;
 
         pwent = getpwnam (username);
         if (pwent == NULL) {
@@ -581,83 +686,19 @@ gs_auth_verify_user (const char       *username,
         /* Initialize PAM. */
         create_pam_handle (username, display, &conv, &status);
         if (status != PAM_SUCCESS) {
-                goto DONE;
+                goto done;
         }
 
         pam_set_item (pam_handle, PAM_USER_PROMPT, _("Username:"));
 
         PAM_NO_DELAY(pam_handle);
 
-        timeout.tv_sec = 0;
-        timeout.tv_nsec = 1;
-        set = block_sigchld ();
-
         did_we_ask_for_password = FALSE;
-        sigtimedwait (&set, NULL, &timeout);
-        unblock_sigchld ();
-
-        if (gs_auth_get_verbose ()) {
-                g_message ("   pam_authenticate (...) ==> %d (%s)",
-                           status,
-                           PAM_STRERROR (pam_handle, status));
+        if (! gs_auth_pam_verify_user (pam_handle, &status)) {
+                goto done;
         }
 
-        if (!gs_auth_identify_user (pam_handle, &status)) {
-                goto DONE;
-        }
-
-        if ((status = pam_get_item (pam_handle, PAM_USER, &p)) != PAM_SUCCESS) {
-                /* is not really an auth problem, but it will
-                   pretty much look as such, it shouldn't really
-                   happen */
-                goto DONE;
-        }
-
-        /* We don't actually care if the account modules fail or succeed,
-         * but we need to run them anyway because certain pam modules
-         * depend on side effects of the account modules getting run.
-         */
-        status2 = pam_acct_mgmt (pam_handle, 0);
-
-        if (gs_auth_get_verbose ()) {
-                g_message ("pam_acct_mgmt (...) ==> %d (%s)\n",
-                           status2,
-                           PAM_STRERROR (pam_handle, status2));
-        }
-
-        /* FIXME: should we handle these? */
-        switch (status2) {
-        case PAM_SUCCESS:
-                break;
-        case PAM_NEW_AUTHTOK_REQD:
-                break;
-        case PAM_AUTHINFO_UNAVAIL:
-                break;
-        case PAM_ACCT_EXPIRED:
-                break;
-        case PAM_PERM_DENIED:
-                break;
-        default :
-                break;
-        }
-
-        /* Each time we successfully authenticate, refresh credentials,
-           for Kerberos/AFS/DCE/etc.  If this fails, just ignore that
-           failure and blunder along; it shouldn't matter.
-
-           Note: this used to be PAM_REFRESH_CRED instead of
-           PAM_REINITIALIZE_CRED, but Jason Heiss <jheiss@ee.washington.edu>
-           says that the Linux PAM library ignores that one, and only refreshes
-           credentials when using PAM_REINITIALIZE_CRED.
-        */
-        status2 = pam_setcred (pam_handle, PAM_REINITIALIZE_CRED);
-        if (gs_auth_get_verbose ()) {
-                g_message ("   pam_setcred (...) ==> %d (%s)",
-                           status2,
-                           PAM_STRERROR (pam_handle, status2));
-        }
-
- DONE:
+ done:
         if (status != PAM_SUCCESS) {
                 set_pam_error (error, status);
         }
